@@ -59,11 +59,18 @@ module Data.Aeson.Types.FromJSON
     -- * Functions
     , fromJSON
     , ifromJSON
+    , typeMismatch
+    , parseField
+    , parseFieldMaybe
+    , parseFieldMaybe'
+    , explicitParseField
+    , explicitParseFieldMaybe
+    , explicitParseFieldMaybe'
+    -- ** Operators
     , (.:)
     , (.:?)
     , (.:!)
     , (.!=)
-    , typeMismatch
 
     -- * Internal
     , parseOptionalFieldWith
@@ -123,6 +130,7 @@ import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
 import qualified Data.Text.Lazy as LT
 import qualified Data.Tree as Tree
+import qualified Data.UUID.Types as UUID
 import qualified Data.Vector as V
 import qualified Data.Vector.Generic as VG
 import qualified Data.Vector.Primitive as VP
@@ -273,14 +281,14 @@ genericLiftParseJSON opts pj pjl = fmap to1 . gParseJSON opts (From1Args pj pjl)
 --
 -- The basic ways to signal a failed conversion are as follows:
 --
--- * 'empty' and 'mzero' work, but are terse and uninformative
+-- * 'empty' and 'mzero' work, but are terse and uninformative;
 --
--- * 'fail' yields a custom error message
+-- * 'fail' yields a custom error message;
 --
 -- * 'typeMismatch' produces an informative message for cases when the
--- value encountered is not of the expected type
+-- value encountered is not of the expected type.
 --
--- An example type and instance:
+-- An example type and instance using 'typeMismatch':
 --
 -- @
 -- \-- Allow ourselves to write 'Text' literals.
@@ -288,15 +296,28 @@ genericLiftParseJSON opts pj pjl = fmap to1 . gParseJSON opts (From1Args pj pjl)
 --
 -- data Coord = Coord { x :: Double, y :: Double }
 --
--- instance FromJSON Coord where
---   parseJSON ('Object' v) = Coord    '<$>'
---                          v '.:' \"x\" '<*>'
---                          v '.:' \"y\"
+-- instance 'FromJSON' Coord where
+--     'parseJSON' ('Object' v) = Coord
+--         '<$>' v '.:' \"x\"
+--         '<*>' v '.:' \"y\"
 --
---   \-- We do not expect a non-'Object' value here.
---   \-- We could use 'mzero' to fail, but 'typeMismatch'
---   \-- gives a much more informative error message.
---   parseJSON invalid    = 'typeMismatch' \"Coord\" invalid
+--     \-- We do not expect a non-'Object' value here.
+--     \-- We could use 'mzero' to fail, but 'typeMismatch'
+--     \-- gives a much more informative error message.
+--     'parseJSON' invalid    = 'typeMismatch' \"Coord\" invalid
+-- @
+--
+-- For this common case of only being concerned with a single
+-- type of JSON value, the functions 'withObject', 'withNumber', etc.
+-- are provided. Their use is to be preferred when possible, since
+-- they are more terse. Using 'withObject', we can rewrite the above instance
+-- (assuming the same language extension and data type) as:
+--
+-- @
+-- instance 'FromJSON' Coord where
+--     'parseJSON' = 'withObject' \"Coord\" $ \v -> Coord
+--         '<$>' v '.:' \"x\"
+--         '<*>' v '.:' \"y\"
 -- @
 --
 -- Instead of manually writing your 'FromJSON' instance, there are two options
@@ -304,7 +325,7 @@ genericLiftParseJSON opts pj pjl = fmap to1 . gParseJSON opts (From1Args pj pjl)
 --
 -- * "Data.Aeson.TH" provides Template Haskell functions which will derive an
 -- instance at compile time. The generated instance is optimized for your type
--- so will probably be more efficient than the following two options:
+-- so it will probably be more efficient than the following option.
 --
 -- * The compiler can provide a default generic implementation for
 -- 'parseJSON'.
@@ -322,18 +343,21 @@ genericLiftParseJSON opts pj pjl = fmap to1 . gParseJSON opts (From1Args pj pjl)
 --
 -- data Coord = Coord { x :: Double, y :: Double } deriving 'Generic'
 --
--- instance FromJSON Coord
+-- instance 'FromJSON' Coord
 -- @
 --
--- If @DefaultSignatures@ doesn't give exactly the results you want,
--- you can customize the generic decoding with only a tiny amount of
--- effort, using 'genericParseJSON' with your preferred 'Options':
+-- The default implementation will be equivalent to
+-- @parseJSON = 'genericParseJSON' 'defaultOptions'@; If you need different
+-- options, you can customize the generic decoding by defining:
 --
 -- @
--- instance FromJSON Coord where
---     parseJSON = 'genericParseJSON' 'defaultOptions'
+-- customOptions = 'defaultOptions'
+--                 { 'fieldLabelModifier' = 'map' 'Data.Char.toUpper'
+--                 }
+--
+-- instance 'FromJSON' Coord where
+--     'parseJSON' = 'genericParseJSON' customOptions
 -- @
-
 class FromJSON a where
     parseJSON :: Value -> Parser a
 
@@ -495,7 +519,7 @@ typeMismatch expected actual =
 --
 -- * "Data.Aeson.TH" provides Template Haskell functions which will derive an
 -- instance at compile time. The generated instance is optimized for your type
--- so will probably be more efficient than the following two options:
+-- so it will probably be more efficient than the following option.
 --
 -- * The compiler can provide a default generic implementation for
 -- 'liftParseJSON'.
@@ -513,16 +537,20 @@ typeMismatch expected actual =
 --
 -- data Pair a b = Pair { pairFst :: a, pairSnd :: b } deriving 'Generic1'
 --
--- instance FromJSON a => FromJSON1 (Pair a)
+-- instance 'FromJSON' a => 'FromJSON1' (Pair a)
 -- @
 --
--- If @DefaultSignatures@ doesn't give exactly the results you want,
+-- If the default implementation doesn't give exactly the results you want,
 -- you can customize the generic decoding with only a tiny amount of
 -- effort, using 'genericLiftParseJSON' with your preferred 'Options':
 --
 -- @
--- instance FromJSON a => FromJSON1 (Pair a) where
---     liftParseJSON = 'genericLiftParseJSON' 'defaultOptions'
+-- customOptions = 'defaultOptions'
+--                 { 'fieldLabelModifier' = 'map' 'Data.Char.toUpper'
+--                 }
+--
+-- instance 'FromJSON' a => 'FromJSON1' (Pair a) where
+--     'liftParseJSON' = 'genericLiftParseJSON' customOptions
 -- @
 class FromJSON1 f where
     liftParseJSON :: (Value -> Parser a) -> (Value -> Parser [a]) -> Value -> Parser (f a)
@@ -593,43 +621,44 @@ instance (FromJSON a) => FromJSON [a] where
 -- Functions
 -------------------------------------------------------------------------------
 
--- | @withObject expected f value@ applies @f@ to the 'Object' when @value@ is an @Object@
---   and fails using @'typeMismatch' expected@ otherwise.
+-- | @'withObject' expected f value@ applies @f@ to the 'Object' when @value@
+--   is an 'Object' and fails using @'typeMismatch' expected@ otherwise.
 withObject :: String -> (Object -> Parser a) -> Value -> Parser a
 withObject _        f (Object obj) = f obj
 withObject expected _ v            = typeMismatch expected v
 {-# INLINE withObject #-}
 
--- | @withText expected f value@ applies @f@ to the 'Text' when @value@ is a @String@
---   and fails using @'typeMismatch' expected@ otherwise.
+-- | @'withText' expected f value@ applies @f@ to the 'Text' when @value@ is a
+--   'String' and fails using @'typeMismatch' expected@ otherwise.
 withText :: String -> (Text -> Parser a) -> Value -> Parser a
 withText _        f (String txt) = f txt
 withText expected _ v            = typeMismatch expected v
 {-# INLINE withText #-}
 
--- | @withArray expected f value@ applies @f@ to the 'Array' when @value@ is an @Array@
---   and fails using @'typeMismatch' expected@ otherwise.
+-- | @'withArray' expected f value@ applies @f@ to the 'Array' when @value@ is
+-- an 'Array' and fails using @'typeMismatch' expected@ otherwise.
 withArray :: String -> (Array -> Parser a) -> Value -> Parser a
 withArray _        f (Array arr) = f arr
 withArray expected _ v           = typeMismatch expected v
 {-# INLINE withArray #-}
 
--- | @withNumber expected f value@ applies @f@ to the 'Number' when @value@ is a 'Number'.
---   and fails using @'typeMismatch' expected@ otherwise.
+-- | @'withNumber' expected f value@ applies @f@ to the 'Number' when @value@
+-- is a 'Number' and fails using @'typeMismatch' expected@ otherwise.
 withNumber :: String -> (Number -> Parser a) -> Value -> Parser a
 withNumber expected f = withScientific expected (f . scientificToNumber)
 {-# INLINE withNumber #-}
 {-# DEPRECATED withNumber "Use withScientific instead" #-}
 
--- | @withScientific expected f value@ applies @f@ to the 'Scientific' number when @value@ is a 'Number'.
---   and fails using @'typeMismatch' expected@ otherwise.
+-- | @'withScientific' expected f value@ applies @f@ to the 'Scientific' number
+-- when @value@ is a 'Number' and fails using @'typeMismatch' expected@
+-- otherwise.
 withScientific :: String -> (Scientific -> Parser a) -> Value -> Parser a
 withScientific _        f (Number scientific) = f scientific
 withScientific expected _ v                   = typeMismatch expected v
 {-# INLINE withScientific #-}
 
--- | @withBool expected f value@ applies @f@ to the 'Bool' when @value@ is a @Bool@
---   and fails using @'typeMismatch' expected@ otherwise.
+-- | @'withBool' expected f value@ applies @f@ to the 'Bool' when @value@ is a
+-- 'Bool' and fails using @'typeMismatch' expected@ otherwise.
 withBool :: String -> (Bool -> Parser a) -> Value -> Parser a
 withBool _        f (Bool arr) = f arr
 withBool expected _ v          = typeMismatch expected v
@@ -653,9 +682,7 @@ ifromJSON = iparse parseJSON
 -- in an object for it to be valid.  If the key and value are
 -- optional, use '.:?' instead.
 (.:) :: (FromJSON a) => Object -> Text -> Parser a
-obj .: key = case H.lookup key obj of
-               Nothing -> fail $ "key " ++ show key ++ " not present"
-               Just v  -> parseJSON v <?> Key key
+(.:) = explicitParseField parseJSON
 {-# INLINE (.:) #-}
 
 -- | Retrieve the value associated with the given key of an 'Object'. The
@@ -666,9 +693,7 @@ obj .: key = case H.lookup key obj of
 -- from an object without affecting its validity.  If the key and
 -- value are mandatory, use '.:' instead.
 (.:?) :: (FromJSON a) => Object -> Text -> Parser (Maybe a)
-obj .:? key = case H.lookup key obj of
-               Nothing -> pure Nothing
-               Just v  -> parseJSON v <?> Key key
+(.:?) = explicitParseFieldMaybe parseJSON
 {-# INLINE (.:?) #-}
 
 -- | Retrieve the value associated with the given key of an 'Object'.
@@ -678,11 +703,47 @@ obj .:? key = case H.lookup key obj of
 -- This differs from '.:?' by attempting to parse 'Null' the same as any
 -- other JSON value, instead of interpreting it as 'Nothing'.
 (.:!) :: (FromJSON a) => Object -> Text -> Parser (Maybe a)
-obj .:! key = case H.lookup key obj of
-               Nothing -> pure Nothing
-               Just v  -> Just <$> parseJSON v <?> Key key
+(.:!) = explicitParseFieldMaybe' parseJSON
 {-# INLINE (.:!) #-}
 
+-- | Function variant of '.:'.
+parseField :: (FromJSON a) => Object -> Text -> Parser a
+parseField = (.:)
+{-# INLINE parseField #-}
+
+-- | Function variant of '.:?'.
+parseFieldMaybe :: (FromJSON a) => Object -> Text -> Parser (Maybe a)
+parseFieldMaybe = (.:?)
+{-# INLINE parseFieldMaybe #-}
+
+-- | Function variant of '.:!'.
+parseFieldMaybe' :: (FromJSON a) => Object -> Text -> Parser (Maybe a)
+parseFieldMaybe' = (.:!)
+{-# INLINE parseFieldMaybe' #-}
+
+-- | Variant of '.:' with explicit parser function.
+--
+-- E.g. @'explicitParseField' 'parseJSON1' :: ('FromJSON1' f, 'FromJSON' a) -> 'Object' -> 'Text' -> 'Parser' (f a)@
+explicitParseField :: (Value -> Parser a) -> Object -> Text -> Parser a
+explicitParseField p obj key = case H.lookup key obj of
+    Nothing -> fail $ "key " ++ show key ++ " not present"
+    Just v  -> p v <?> Key key
+{-# INLINE explicitParseField #-}
+
+-- | Variant of '.:?' with explicit parser function.
+explicitParseFieldMaybe :: (Value -> Parser a) -> Object -> Text -> Parser (Maybe a)
+explicitParseFieldMaybe p obj key = case H.lookup key obj of
+    Nothing -> pure Nothing
+    Just v  -> liftParseJSON p (listParser p) v <?> Key key -- listParser isn't used by maybe instance.
+{-# INLINE explicitParseFieldMaybe #-}
+
+-- | Variant of '.:!' with explicit parser function.
+explicitParseFieldMaybe' :: (Value -> Parser a) -> Object -> Text -> Parser (Maybe a)
+explicitParseFieldMaybe' p obj key = case H.lookup key obj of
+    Nothing -> pure Nothing
+    Just v  -> Just <$> p v <?> Key key
+{-# INLINE explicitParseFieldMaybe' #-}
+
 -- | Helper for use in combination with '.:?' to provide default
 -- values for optional JSON object fields.
 --
@@ -1360,7 +1421,7 @@ instance (FromJSON a) => FromJSON (Identity a) where
     parseJSONList = liftParseJSONList parseJSON parseJSONList
     {-# INLINE parseJSONList #-}
 
-instance (FromJSONKey a, FromJSON a) => FromJSONKey (Identity a) where
+instance (FromJSONKey a) => FromJSONKey (Identity a) where
     fromJSONKey = coerceFromJSONKeyFunction (fromJSONKey :: FromJSONKeyFunction a)
     fromJSONKeyList = coerceFromJSONKeyFunction (fromJSONKeyList :: FromJSONKeyFunction [a])
 
@@ -1486,6 +1547,18 @@ instance (FromJSON v) => FromJSON (Tree.Tree v) where
     parseJSON = parseJSON1
     {-# INLINE parseJSON #-}
 
+-------------------------------------------------------------------------------
+-- uuid
+-------------------------------------------------------------------------------
+
+instance FromJSON UUID.UUID where
+    parseJSON = withText "UUID" $
+        maybe (fail "Invalid UUID") pure . UUID.fromText
+
+instance FromJSONKey UUID.UUID where
+    fromJSONKey = FromJSONKeyTextParser $
+        maybe (fail "Invalid UUID") pure . UUID.fromText
+
 -------------------------------------------------------------------------------
 -- vector
 -------------------------------------------------------------------------------
@@ -1725,12 +1798,21 @@ instance FromJSON a => FromJSON (Semigroup.Option a) where
 -- tagged
 -------------------------------------------------------------------------------
 
+instance FromJSON1 Proxy where
+    {-# INLINE liftParseJSON #-}
+    liftParseJSON _ _ Null = pure Proxy
+    liftParseJSON _ _ v    = typeMismatch "Proxy" v
+
 instance FromJSON (Proxy a) where
     {-# INLINE parseJSON #-}
     parseJSON Null = pure Proxy
     parseJSON v    = typeMismatch "Proxy" v
 
 
+instance FromJSON2 Tagged where
+    liftParseJSON2 _ _ p _ = fmap Tagged . p
+    {-# INLINE liftParseJSON2 #-}
+
 instance FromJSON1 (Tagged a) where
     liftParseJSON p _ = fmap Tagged . p
     {-# INLINE liftParseJSON #-}
