@@ -2,7 +2,6 @@
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE Rank2Types #-}
-{-# LANGUAGE RecordWildCards #-}
 #if __GLASGOW_HASKELL__ >= 800
 -- a) THQ works on cross-compilers and unregisterised GHCs
 -- b) may make compilation faster as no dynamic loading is ever needed (not sure about this)
@@ -75,6 +74,7 @@ import Data.Data (Data)
 import Data.Foldable (foldl')
 import Data.HashMap.Strict (HashMap)
 import Data.Hashable (Hashable(..))
+import Data.List (intercalate)
 import Data.Scientific (Scientific)
 import Data.Semigroup (Semigroup((<>)))
 import Data.String (IsString(..))
@@ -527,16 +527,17 @@ data Options = Options
     }
 
 instance Show Options where
-  show Options{..} = "Options {" ++
-    "fieldLabelModifier =~ " ++
-      show (fieldLabelModifier "exampleField") ++ ", " ++
-    "constructorTagModifier =~ " ++
-      show (constructorTagModifier "ExampleConstructor") ++ ", " ++
-    "allNullaryToStringTag = " ++ show allNullaryToStringTag ++ ", " ++
-    "omitNothingFields = " ++ show omitNothingFields ++ ", " ++
-    "sumEncoding = " ++ show sumEncoding ++ ", " ++
-    "unwrapUnaryRecords = " ++ show unwrapUnaryRecords ++
-    "}"
+  show (Options f c a o s u) =
+       "Options {"
+    ++ intercalate ", "
+      [ "fieldLabelModifier =~ " ++ show (f "exampleField")
+      , "constructorTagModifier =~ " ++ show (c "ExampleConstructor")
+      , "allNullaryToStringTag = " ++ show a
+      , "omitNothingFields = " ++ show o
+      , "sumEncoding = " ++ show s
+      , "unwrapUnaryRecords = " ++ show u
+      ]
+    ++ "}"
 
 -- | Specifies how to encode constructors of a sum datatype.
 data SumEncoding =
@@ -554,18 +555,18 @@ data SumEncoding =
     -- the 'contentsFieldName' field.
   | UntaggedValue
     -- ^ Constructor names won't be encoded. Instead only the contents of the
-    -- constructor will be encoded as if the type had single constructor. JSON
+    -- constructor will be encoded as if the type had a single constructor. JSON
     -- encodings have to be disjoint for decoding to work properly.
     --
     -- When decoding, constructors are tried in the order of definition. If some
     -- encodings overlap, the first one defined will succeed.
     --
-    -- /Note:/ Nullary constructors are encoded as the string (using
+    -- /Note:/ Nullary constructors are encoded as strings (using
     -- 'constructorTagModifier'). Having a nullary constructor alongside a
     -- single field constructor that encodes to a string leads to ambiguity.
     --
     -- /Note:/ Only the last error is kept when decoding, so in the case of
-    -- mailformed JSON, only an error for the last constructor will be reported.
+    -- malformed JSON, only an error for the last constructor will be reported.
   | ObjectWithSingleField
     -- ^ A constructor will be encoded to an object with a single
     -- field named after the constructor tag (modified by the
