@@ -1,4 +1,3 @@
-{-# LANGUAGE BangPatterns #-}
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE DataKinds #-}
 {-# LANGUAGE DefaultSignatures #-}
@@ -113,6 +112,7 @@ import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
 import qualified Data.Text.Lazy as LT
 import qualified Data.Tree as Tree
+import qualified Data.UUID.Types as UUID
 import qualified Data.Vector as V
 import qualified Data.Vector.Generic as VG
 import qualified Data.Vector.Mutable as VM
@@ -120,6 +120,9 @@ import qualified Data.Vector.Primitive as VP
 import qualified Data.Vector.Storable as VS
 import qualified Data.Vector.Unboxed as VU
 
+import qualified Data.Aeson.Encoding.Builder as EB
+import qualified Data.ByteString.Builder as B
+
 #if !(MIN_VERSION_bytestring(0,10,0))
 import Foreign.ForeignPtr (withForeignPtr)
 import Foreign.Marshal.Utils (copyBytes)
@@ -128,6 +131,8 @@ import qualified Data.ByteString.Internal as S
 import qualified Data.ByteString.Lazy.Internal as L
 #endif
 
+{-# ANN module ("HLint: ignore Reduce duplication"::String) #-}
+
 toJSONPair :: (a -> Value) -> (b -> Value) -> (a, b) -> Value
 toJSONPair a b = liftToJSON2 a (listValue a) b (listValue b)
 {-# INLINE toJSONPair #-}
@@ -201,6 +206,9 @@ genericLiftToEncoding opts te tel = gToEncoding opts (To1Args te tel) . from1
 
 -- | A type that can be converted to JSON.
 --
+-- Instances in general /must/ specify 'toJSON' and /should/ (but don't need
+-- to) specify 'toEncoding'.
+--
 -- An example type and instance:
 --
 -- @
@@ -209,10 +217,10 @@ genericLiftToEncoding opts te tel = gToEncoding opts (To1Args te tel) . from1
 --
 -- data Coord = Coord { x :: Double, y :: Double }
 --
--- instance ToJSON Coord where
---   toJSON (Coord x y) = 'object' [\"x\" '.=' x, \"y\" '.=' y]
+-- instance 'ToJSON' Coord where
+--   'toJSON' (Coord x y) = 'object' [\"x\" '.=' x, \"y\" '.=' y]
 --
---   toEncoding (Coord x y) = 'pairs' (\"x\" '.=' x '<>' \"y\" '.=' y)
+--   'toEncoding' (Coord x y) = 'pairs' (\"x\" '.=' x '<>' \"y\" '.=' y)
 -- @
 --
 -- Instead of manually writing your 'ToJSON' instance, there are two options
@@ -220,17 +228,15 @@ genericLiftToEncoding opts te tel = gToEncoding opts (To1Args te tel) . from1
 --
 -- * "Data.Aeson.TH" provides Template Haskell functions which will derive an
 -- instance at compile time. The generated instance is optimized for your type
--- so will probably be more efficient than the following two options:
+-- so it will probably be more efficient than the following option.
 --
 -- * The compiler can provide a default generic implementation for
 -- 'toJSON'.
 --
 -- To use the second, simply add a @deriving 'Generic'@ clause to your
--- datatype and declare a 'ToJSON' instance for your datatype without giving
--- definitions for 'toJSON' or 'toEncoding'.
---
--- For example, the previous example can be simplified to a more
--- minimal instance:
+-- datatype and declare a 'ToJSON' instance. If you require nothing other than
+-- 'defaultOptions', it is sufficient to write (and this is the only
+-- alternative where the default 'toJSON' implementation is sufficient):
 --
 -- @
 -- {-\# LANGUAGE DeriveGeneric \#-}
@@ -239,30 +245,41 @@ genericLiftToEncoding opts te tel = gToEncoding opts (To1Args te tel) . from1
 --
 -- data Coord = Coord { x :: Double, y :: Double } deriving 'Generic'
 --
--- instance ToJSON Coord where
---     toEncoding = 'genericToEncoding' 'defaultOptions'
+-- instance 'ToJSON' Coord where
+--     'toEncoding' = 'genericToEncoding' 'defaultOptions'
 -- @
 --
--- Why do we provide an implementation for 'toEncoding' here?  The
--- 'toEncoding' function is a relatively new addition to this class.
--- To allow users of older versions of this library to upgrade without
--- having to edit all of their instances or encounter surprising
--- incompatibilities, the default implementation of 'toEncoding' uses
--- 'toJSON'.  This produces correct results, but since it performs an
--- intermediate conversion to a 'Value', it will be less efficient
--- than directly emitting an 'Encoding'.  Our one-liner definition of
--- 'toEncoding' above bypasses the intermediate 'Value'.
---
--- If @DefaultSignatures@ doesn't give exactly the results you want,
--- you can customize the generic encoding with only a tiny amount of
--- effort, using 'genericToJSON' and 'genericToEncoding' with your
--- preferred 'Options':
+-- If on the other hand you wish to customize the generic decoding, you have
+-- to implement both methods:
 --
 -- @
--- instance ToJSON Coord where
---     toJSON     = 'genericToJSON' 'defaultOptions'
---     toEncoding = 'genericToEncoding' 'defaultOptions'
+-- customOptions = 'defaultOptions'
+--                 { 'fieldLabelModifier' = 'map' 'Data.Char.toUpper'
+--                 }
+--
+-- instance 'ToJSON' Coord where
+--     'toJSON'     = 'genericToJSON' customOptions
+--     'toEncoding' = 'genericToEncoding' customOptions
 -- @
+--
+-- Previous versions of this library only had the 'toJSON' method. Adding
+-- 'toEncoding' had to reasons:
+--
+-- 1. toEncoding is more efficient for the common case that the output of
+-- 'toJSON' is directly serialized to a @ByteString@.
+-- Further, expressing either method in terms of the other would be
+-- non-optimal.
+--
+-- 2. The choice of defaults allows a smooth transition for existing users:
+-- Existing instances that do not define 'toEncoding' still
+-- compile and have the correct semantics. This is ensured by making
+-- the default implementation of 'toEncoding' use 'toJSON'. This produces
+-- correct results, but since it performs an intermediate conversion to a
+-- 'Value', it will be less efficient than directly emitting an 'Encoding'.
+-- (this also means that specifying nothing more than
+-- @instance ToJSON Coord@ would be sufficient as a generically decoding
+-- instance, but there probably exists no good reason to not specify
+-- 'toEncoding' in new instances.)
 class ToJSON a where
     -- | Convert a Haskell value to a JSON-friendly intermediate type.
     toJSON     :: a -> Value
@@ -284,8 +301,8 @@ class ToJSON a where
     -- extension, and then have GHC generate a method body as follows.
     --
     -- @
-    -- instance ToJSON Coord where
-    --     toEncoding = 'genericToEncoding' 'defaultOptions'
+    -- instance 'ToJSON' Coord where
+    --     'toEncoding' = 'genericToEncoding' 'defaultOptions'
     -- @
 
     toEncoding :: a -> Encoding
@@ -487,7 +504,7 @@ contramapToJSONKeyFunction h x = case x of
 --
 -- * "Data.Aeson.TH" provides Template Haskell functions which will derive an
 -- instance at compile time. The generated instance is optimized for your type
--- so will probably be more efficient than the following two options:
+-- so it will probably be more efficient than the following option.
 --
 -- * The compiler can provide a default generic implementation for
 -- 'toJSON1'.
@@ -505,19 +522,25 @@ contramapToJSONKeyFunction h x = case x of
 --
 -- data Pair = Pair { pairFst :: a, pairSnd :: b } deriving 'Generic1'
 --
--- instance ToJSON a => ToJSON1 (Pair a)
+-- instance 'ToJSON' a => 'ToJSON1' (Pair a)
 -- @
 --
--- If @DefaultSignatures@ doesn't give exactly the results you want,
+-- If the default implementation doesn't give exactly the results you want,
 -- you can customize the generic encoding with only a tiny amount of
 -- effort, using 'genericLiftToJSON' and 'genericLiftToEncoding' with
 -- your preferred 'Options':
 --
 -- @
--- instance ToJSON a => ToJSON1 (Pair a) where
---     liftToJSON     = 'genericLiftToJSON' 'defaultOptions'
---     liftToEncoding = 'genericLiftToEncoding' 'defaultOptions'
+-- customOptions = 'defaultOptions'
+--                 { 'fieldLabelModifier' = 'map' 'Data.Char.toUpper'
+--                 }
+--
+-- instance 'ToJSON' a => 'ToJSON1' (Pair a) where
+--     'liftToJSON'     = 'genericLiftToJSON' customOptions
+--     'liftToEncoding' = 'genericLiftToEncoding' customOptions
 -- @
+--
+-- See also 'ToJSON'.
 class ToJSON1 f where
     liftToJSON :: (a -> Value) -> ([a] -> Value) -> f a -> Value
 
@@ -584,7 +607,7 @@ toEncoding2 = liftToEncoding2 toEncoding toEncodingList toEncoding toEncodingLis
 -- @
 -- newtype F a = F [a]
 --
--- -- This instance encodes String as an array of chars
+-- -- This instance encodes 'String' as an array of chars
 -- instance 'ToJSON1' F where
 --     'liftToJSON'     tj _ (F xs) = 'liftToJSON'     tj ('listValue'    tj) xs
 --     'liftToEncoding' te _ (F xs) = 'liftToEncoding' te ('listEncoding' te) xs
@@ -615,6 +638,11 @@ instance ToJSON1 [] where
     {-# INLINE liftToEncoding #-}
 
 instance (ToJSON a) => ToJSON [a] where
+    {-# SPECIALIZE instance ToJSON String #-}
+    {-# SPECIALIZE instance ToJSON [String] #-}
+    {-# SPECIALIZE instance ToJSON [Array] #-}
+    {-# SPECIALIZE instance ToJSON [Object] #-}
+
     toJSON = toJSON1
     {-# INLINE toJSON #-}
 
@@ -1661,7 +1689,7 @@ instance (ToJSON a) => ToJSON (Identity a) where
     toEncodingList = liftToEncodingList toEncoding toEncodingList
     {-# INLINE toEncodingList #-}
 
-instance (ToJSONKey a, ToJSON a) => ToJSONKey (Identity a) where
+instance (ToJSONKey a) => ToJSONKey (Identity a) where
     toJSONKey = contramapToJSONKeyFunction runIdentity toJSONKey
     toJSONKeyList = contramapToJSONKeyFunction (map runIdentity) toJSONKeyList
 
@@ -1852,6 +1880,17 @@ instance (ToJSON v) => ToJSON (Tree.Tree v) where
     toEncoding = toEncoding1
     {-# INLINE toEncoding #-}
 
+-------------------------------------------------------------------------------
+-- uuid
+-------------------------------------------------------------------------------
+
+instance ToJSON UUID.UUID where
+    toJSON = toJSON . UUID.toText
+    toEncoding = E.unsafeToEncoding . EB.quote . B.byteString . UUID.toASCIIBytes
+
+instance ToJSONKey UUID.UUID where
+    toJSONKey = ToJSONKeyText UUID.toText $
+        E.unsafeToEncoding . EB.quote . B.byteString . UUID.toASCIIBytes
 
 -------------------------------------------------------------------------------
 -- vector
@@ -1865,6 +1904,8 @@ instance ToJSON1 Vector where
     {-# INLINE liftToEncoding #-}
 
 instance (ToJSON a) => ToJSON (Vector a) where
+    {-# SPECIALIZE instance ToJSON Array #-}
+
     toJSON = toJSON1
     {-# INLINE toJSON #-}
 
@@ -1936,6 +1977,8 @@ instance ToJSONKey k => ToJSON1 (H.HashMap k) where
     {-# INLINE liftToEncoding #-}
 
 instance (ToJSON v, ToJSONKey k) => ToJSON (H.HashMap k v) where
+    {-# SPECIALIZE instance ToJSON Object #-}
+
     toJSON = toJSON1
     {-# INLINE toJSON #-}
 
@@ -2165,6 +2208,13 @@ instance ToJSON a => ToJSON (Semigroup.Option a) where
 -- tagged
 -------------------------------------------------------------------------------
 
+instance ToJSON1 Proxy where
+    liftToJSON _ _ _ = Null
+    {-# INLINE liftToJSON #-}
+
+    liftToEncoding _ _ _ = E.null_
+    {-# INLINE liftToEncoding #-}
+
 instance ToJSON (Proxy a) where
     toJSON _ = Null
     {-# INLINE toJSON #-}
@@ -2173,6 +2223,13 @@ instance ToJSON (Proxy a) where
     {-# INLINE toEncoding #-}
 
 
+instance ToJSON2 Tagged where
+    liftToJSON2 _ _ t _ (Tagged x) = t x
+    {-# INLINE liftToJSON2 #-}
+
+    liftToEncoding2 _ _ t _ (Tagged x) = t x
+    {-# INLINE liftToEncoding2 #-}
+
 instance ToJSON1 (Tagged a) where
     liftToJSON t _ (Tagged x) = t x
     {-# INLINE liftToJSON #-}
@@ -2187,7 +2244,6 @@ instance ToJSON b => ToJSON (Tagged a b) where
     toEncoding = toEncoding1
     {-# INLINE toEncoding #-}
 
-
 instance ToJSONKey b => ToJSONKey (Tagged a b) where
     toJSONKey = contramapToJSONKeyFunction unTagged toJSONKey
     toJSONKeyList = contramapToJSONKeyFunction (fmap unTagged) toJSONKeyList
@@ -2769,11 +2825,11 @@ lazyToStrictByteString = packChunks
 
 -- | Pack the chunks of a lazy bytestring into a single strict bytestring.
 packChunks :: L.ByteString -> S.ByteString
-packChunks lbs = do
+packChunks lbs =
     S.unsafeCreate (fromIntegral $ L.length lbs) (copyChunks lbs)
   where
-    copyChunks !L.Empty                         !_pf = return ()
-    copyChunks !(L.Chunk (S.PS fpbuf o l) lbs') !pf  = do
+    copyChunks L.Empty                         _pf = return ()
+    copyChunks (L.Chunk (S.PS fpbuf o l) lbs') pf  = do
         withForeignPtr fpbuf $ \pbuf ->
             copyBytes pf (pbuf `plusPtr` o) l
         copyChunks lbs' (pf `plusPtr` l)
