@@ -1,7 +1,11 @@
+{-# LANGUAGE PackageImports #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 
 module Main (main) where
 
+import Prelude ()
+import Prelude.Compat
+
 import Compare.BufferBuilder ()
 import Compare.JsonBuilder ()
 import Criterion.Main
@@ -10,8 +14,8 @@ import Data.Json.Builder
 import Twitter
 import Twitter.Manual ()
 import Typed.Common
+import qualified "aeson-benchmarks" Data.Aeson as Aeson
 import qualified Compare.JsonBench as JsonBench
-import qualified Data.Aeson as Aeson
 
 main :: IO ()
 main =
