@@ -1,11 +1,13 @@
 module Main (main) where
 
+import Prelude ()
+import Prelude.Compat
+
 import Criterion.Main
-import qualified Data.Aeson.Encode.Builder as AB
+import qualified Data.Aeson.Encoding.Builder as AB
 import qualified Data.ByteString.Builder as B
 import qualified Data.Text as T
 
-
 main :: IO ()
 main = do
   let txt = "append (append b (primBounded w1 x1)) (primBounded w2 x2)"
