@@ -20,7 +20,7 @@ import qualified Data.ByteString as B
 main = do
   (cnt:args) <- getArgs
   let count = read cnt :: Int
-  forM_ args $ \arg -> bracket (openFile arg ReadMode) hClose $ \h -> do
+  forM_ args $ \arg -> withFile arg ReadMode $ \h -> do
     putStrLn $ arg ++ ":"
     start <- getCurrentTime
     let loop !n
