@@ -1,5 +1,8 @@
 module Main (main) where
 
+import Prelude ()
+import Prelude.Compat
+
 import Criterion.Main
 import qualified Typed.Generic as Generic
 import qualified Typed.Manual as Manual
