@@ -4,14 +4,15 @@
 {-# LANGUAGE TypeFamilies #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
-module DataFamilies.Instances where
+module DataFamilies.Instances () where
+
+import Prelude ()
+import Prelude.Compat
 
-import Control.Applicative
 import Data.Aeson.TH
 import Data.Aeson.Types (FromJSON(..))
 import DataFamilies.Types
 import Test.QuickCheck (Arbitrary(..), elements, oneof)
-import Prelude
 
 instance (Arbitrary a) => Arbitrary (Approx a) where
     arbitrary = Approx <$> arbitrary
@@ -24,6 +25,7 @@ instance Arbitrary a => Arbitrary (SomeType c () a) where
                       , Unary   <$> arbitrary
                       , Product <$> arbitrary <*> arbitrary <*> arbitrary
                       , Record  <$> arbitrary <*> arbitrary <*> arbitrary
+                      , List    <$> arbitrary
                       ]
 
 instance Arbitrary (GADT String) where
