@@ -1,11 +1,19 @@
+-- DataKinds is needed for deriveAll0 calls on GHC 8
+{-# LANGUAGE DataKinds #-}
+{-# LANGUAGE DeriveGeneric #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE GADTs #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE StandaloneDeriving #-}
+{-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE TypeFamilies #-}
 
 module DataFamilies.Types where
 
+import Prelude ()
+import Prelude.Compat
+
+import Generics.Deriving.TH (deriveAll0)
 import Types (ApproxEq(..))
 
 data family Nullary a
@@ -19,7 +27,9 @@ data instance SomeType c () a = Nullary
                               | Record { testOne   :: Double
                                        , testTwo   :: Maybe Bool
                                        , testThree :: Maybe a
-                                       } deriving (Eq, Show)
+                                       }
+                              | List [a]
+    deriving (Eq, Show)
 
 data family Approx a
 newtype instance Approx a = Approx { fromApprox :: a }
@@ -34,3 +44,11 @@ data instance GADT a where
 
 deriving instance Eq   (GADT a)
 deriving instance Show (GADT a)
+
+-- We use generic-deriving to be able to derive Generic instances for
+-- data families on GHC 7.4.
+
+$(deriveAll0 'C1)
+$(deriveAll0 'C4)
+$(deriveAll0 'Approx)
+$(deriveAll0 'Nullary)
