@@ -1,4 +1,11 @@
-module Functions where
+module Functions
+    (
+      approxEq
+    , approxEqWith
+    ) where
+
+import Prelude ()
+import Prelude.Compat
 
 approxEq :: (Fractional a, Ord a) => a -> a -> Bool
 approxEq = approxEqWith 1e-15 1e-15
