@@ -3,7 +3,6 @@
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RecordWildCards #-}
-{-# LANGUAGE StandaloneDeriving #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
 module Instances () where
@@ -25,11 +24,12 @@ import Test.QuickCheck (Arbitrary(..), elements,  oneof)
 import Types
 import qualified Data.DList as DList
 import qualified Data.HashMap.Strict as HM
+import qualified Data.UUID.Types as UUID
 
 #if !MIN_VERSION_QuickCheck(2,9,0)
 import Control.Applicative (Const(..))
-import Data.List.NonEmpty (NonEmpty(..))
 import Data.Functor.Identity (Identity (..))
+import Data.List.NonEmpty (NonEmpty(..))
 import Data.Version
 import Test.QuickCheck (getNonNegative, listOf1, resize)
 #endif
@@ -44,6 +44,8 @@ import Data.Hashable.Time ()
 import Numeric.Natural
 #endif
 
+{-# ANN module ("HLint: ignore Use fewer imports"::String) #-}
+
 -- "System" types.
 
 instance Arbitrary DotNetTime where
@@ -218,3 +220,10 @@ makeVersion b = Version b []
 instance Arbitrary a => Arbitrary (Identity a) where
     arbitrary = Identity <$> arbitrary
 #endif
+
+instance Arbitrary UUID.UUID where
+    arbitrary = UUID.fromWords
+        <$> arbitrary
+        <*> arbitrary
+        <*> arbitrary
+        <*> arbitrary
