@@ -33,7 +33,7 @@ import Instances ()
 import Numeric.Natural (Natural)
 import Test.Framework (Test, testGroup)
 import Test.Framework.Providers.QuickCheck2 (testProperty)
-import Test.QuickCheck (Arbitrary(..), Property, (===), (.&&.), counterexample)
+import Test.QuickCheck (Arbitrary(..), Property, Testable, (===), (.&&.), counterexample)
 import Types
 import qualified Data.Attoparsec.Lazy as L
 import qualified Data.ByteString.Lazy.Char8 as L
@@ -41,6 +41,7 @@ import qualified Data.HashMap.Strict as H
 import qualified Data.Map as Map
 import qualified Data.Text as T
 import qualified Data.Text.Lazy as LT
+import qualified Data.UUID.Types as UUID
 import qualified Data.Vector as V
 
 encodeDouble :: Double -> Double -> Property
@@ -148,6 +149,11 @@ type S4 = Sum4 Int8 ZonedTime T.Text (Map.Map String Int)
 -- Value properties
 --------------------------------------------------------------------------------
 
+-- | Add the formatted @Value@ to the printed counterexample when the property
+-- fails.
+checkValue :: Testable a => (Value -> a) -> Value -> Property
+checkValue prop v = counterexample (L.unpack (encode v)) (prop v)
+
 isString :: Value -> Bool
 isString (String _) = True
 isString _          = False
@@ -181,6 +187,12 @@ isUntaggedValueETI (Number _) = True
 isUntaggedValueETI (Array a)  = length a == 2
 isUntaggedValueETI _          = False
 
+isEmptyArray :: Value -> Property
+isEmptyArray = checkValue isEmptyArray'
+
+isEmptyArray' :: Value -> Bool
+isEmptyArray' = (Array mempty ==)
+
 
 --------------------------------------------------------------------------------
 
@@ -202,6 +214,7 @@ tests = testGroup "properties" [
     , testProperty "Lazy Text" $ roundTripEq LT.empty
     , testProperty "Foo" $ roundTripEq (undefined :: Foo)
     , testProperty "Day" $ roundTripEq (undefined :: Day)
+    , testProperty "BCE Day" $ roundTripEq (undefined :: BCEDay)
     , testProperty "DotNetTime" $ roundTripEq (undefined :: Approx DotNetTime)
     , testProperty "LocalTime" $ roundTripEq (undefined :: LocalTime)
     , testProperty "TimeOfDay" $ roundTripEq (undefined :: TimeOfDay)
@@ -217,40 +230,41 @@ tests = testGroup "properties" [
     , testProperty "Seq" $ roundTripEq (undefined :: Seq Int)
     , testProperty "Rational" $ roundTripEq (undefined :: Rational)
     , testProperty "Ratio Int" $ roundTripEq (undefined :: Ratio Int)
+    , testProperty "UUID" $ roundTripEq UUID.nil
     , testGroup "functors"
       [ testProperty "Identity Char" $ roundTripEq (undefined :: I Int)
 
       , testProperty "Identity Char" $ roundTripEq (undefined :: I Char)
-      , testProperty "Identity [Char]" $ roundTripEq (undefined :: I [Char])
+      , testProperty "Identity [Char]" $ roundTripEq (undefined :: I String)
       , testProperty "[Identity Char]" $ roundTripEq (undefined :: [I Char])
 
-      , testProperty "Compose I  I  Int" $ roundTripEq (undefined :: Compose I  I  Int)
-      , testProperty "Compose [] I  Int" $ roundTripEq (undefined :: Compose [] I  Int)
-      , testProperty "Compose I  [] Int" $ roundTripEq (undefined :: Compose I  [] Int)
-      , testProperty "Compose [] [] Int" $ roundTripEq (undefined :: Compose [] [] Int)
+      , testProperty "Compose I  I  Int" $ roundTripEq (undefined :: LogScaled (Compose I  I  Int))
+      , testProperty "Compose [] I  Int" $ roundTripEq (undefined :: LogScaled (Compose [] I  Int))
+      , testProperty "Compose I  [] Int" $ roundTripEq (undefined :: LogScaled (Compose I  [] Int))
+      , testProperty "Compose [] [] Int" $ roundTripEq (undefined :: LogScaled (Compose [] [] Int))
 
-      , testProperty "Compose I  I  Char" $ roundTripEq (undefined :: Compose I  I  Char)
-      , testProperty "Compose [] I  Char" $ roundTripEq (undefined :: Compose [] I  Char)
-      , testProperty "Compose I  [] Char" $ roundTripEq (undefined :: Compose I  [] Char)
-      , testProperty "Compose [] [] Char" $ roundTripEq (undefined :: Compose [] [] Char)
+      , testProperty "Compose I  I  Char" $ roundTripEq (undefined :: LogScaled (Compose I  I  Char))
+      , testProperty "Compose [] I  Char" $ roundTripEq (undefined :: LogScaled (Compose [] I  Char))
+      , testProperty "Compose I  [] Char" $ roundTripEq (undefined :: LogScaled (Compose I  [] Char))
+      , testProperty "Compose [] [] Char" $ roundTripEq (undefined :: LogScaled (Compose [] [] Char))
 
-      , testProperty "Compose3 I  I  I  Char" $ roundTripEq (undefined :: Compose3 I  I  I  Char)
-      , testProperty "Compose3 I  [] I  Char" $ roundTripEq (undefined :: Compose3 I  [] I  Char)
-      , testProperty "Compose3 I  I  [] Char" $ roundTripEq (undefined :: Compose3 I  I  [] Char)
-      , testProperty "Compose3 I  [] [] Char" $ roundTripEq (undefined :: Compose3 I  [] [] Char)
-      , testProperty "Compose3 [] I  I  Char" $ roundTripEq (undefined :: Compose3 [] I  I  Char)
-      , testProperty "Compose3 [] [] I  Char" $ roundTripEq (undefined :: Compose3 [] [] I  Char)
-      , testProperty "Compose3 [] I  [] Char" $ roundTripEq (undefined :: Compose3 [] I  [] Char)
-      , testProperty "Compose3 [] [] [] Char" $ roundTripEq (undefined :: Compose3 [] [] [] Char)
+      , testProperty "Compose3 I  I  I  Char" $ roundTripEq (undefined :: LogScaled (Compose3 I  I  I  Char))
+      , testProperty "Compose3 I  [] I  Char" $ roundTripEq (undefined :: LogScaled (Compose3 I  [] I  Char))
+      , testProperty "Compose3 I  I  [] Char" $ roundTripEq (undefined :: LogScaled (Compose3 I  I  [] Char))
+      , testProperty "Compose3 I  [] [] Char" $ roundTripEq (undefined :: LogScaled (Compose3 I  [] [] Char))
+      , testProperty "Compose3 [] I  I  Char" $ roundTripEq (undefined :: LogScaled (Compose3 [] I  I  Char))
+      , testProperty "Compose3 [] [] I  Char" $ roundTripEq (undefined :: LogScaled (Compose3 [] [] I  Char))
+      , testProperty "Compose3 [] I  [] Char" $ roundTripEq (undefined :: LogScaled (Compose3 [] I  [] Char))
+      , testProperty "Compose3 [] [] [] Char" $ roundTripEq (undefined :: LogScaled (Compose3 [] [] [] Char))
 
-      , testProperty "Compose3' I  I  I  Char" $ roundTripEq (undefined :: Compose3' I  I  I  Char)
-      , testProperty "Compose3' I  [] I  Char" $ roundTripEq (undefined :: Compose3' I  [] I  Char)
-      , testProperty "Compose3' I  I  [] Char" $ roundTripEq (undefined :: Compose3' I  I  [] Char)
-      , testProperty "Compose3' I  [] [] Char" $ roundTripEq (undefined :: Compose3' I  [] [] Char)
-      , testProperty "Compose3' [] I  I  Char" $ roundTripEq (undefined :: Compose3' [] I  I  Char)
-      , testProperty "Compose3' [] [] I  Char" $ roundTripEq (undefined :: Compose3' [] [] I  Char)
-      , testProperty "Compose3' [] I  [] Char" $ roundTripEq (undefined :: Compose3' [] I  [] Char)
-      , testProperty "Compose3' [] [] [] Char" $ roundTripEq (undefined :: Compose3' [] [] [] Char)
+      , testProperty "Compose3' I  I  I  Char" $ roundTripEq (undefined :: LogScaled (Compose3' I  I  I  Char))
+      , testProperty "Compose3' I  [] I  Char" $ roundTripEq (undefined :: LogScaled (Compose3' I  [] I  Char))
+      , testProperty "Compose3' I  I  [] Char" $ roundTripEq (undefined :: LogScaled (Compose3' I  I  [] Char))
+      , testProperty "Compose3' I  [] [] Char" $ roundTripEq (undefined :: LogScaled (Compose3' I  [] [] Char))
+      , testProperty "Compose3' [] I  I  Char" $ roundTripEq (undefined :: LogScaled (Compose3' [] I  I  Char))
+      , testProperty "Compose3' [] [] I  Char" $ roundTripEq (undefined :: LogScaled (Compose3' [] [] I  Char))
+      , testProperty "Compose3' [] I  [] Char" $ roundTripEq (undefined :: LogScaled (Compose3' [] I  [] Char))
+      , testProperty "Compose3' [] [] [] Char" $ roundTripEq (undefined :: LogScaled (Compose3' [] [] [] Char))
       ]
     , testGroup "ghcGenerics" [
         testProperty "OneConstructor" $ roundTripEq OneConstructor
@@ -264,7 +278,7 @@ tests = testGroup "properties" [
     , testProperty "Text" $ roundTripKey (undefined :: T.Text)
     , testProperty "String" $ roundTripKey (undefined :: String)
     , testProperty "Int" $ roundTripKey (undefined :: Int)
-    , testProperty "[Text]" $ roundTripKey (undefined :: [T.Text])
+    , testProperty "[Text]" $ roundTripKey (undefined :: LogScaled [T.Text])
     , testProperty "(Int,Char)" $ roundTripKey (undefined :: (Int,Char))
     , testProperty "Integer" $ roundTripKey (undefined :: Integer)
     , testProperty "Natural" $ roundTripKey (undefined :: Natural)
@@ -278,6 +292,7 @@ tests = testGroup "properties" [
 #endif
     , testProperty "Version" $ roundTripKey (undefined :: Version)
     , testProperty "Lazy Text" $ roundTripKey (undefined :: LT.Text)
+    , testProperty "UUID" $ roundTripKey UUID.nil
     ]
   , testGroup "toFromJSON" [
       testProperty "Integer" (toFromJSON :: Integer -> Property)
@@ -406,21 +421,27 @@ tests = testGroup "properties" [
           , testProperty "ObjectWithSingleField unary" (toParseJSON1 thSomeTypeLiftParseJSONObjectWithSingleField thSomeTypeLiftToJSONObjectWithSingleField)
 
           ]
-       , testGroup "Approx" [
-            testProperty "string"                (isString                . thApproxToJSONUnwrap)
-          , testProperty "ObjectWithSingleField" (isObjectWithSingleField . thApproxToJSONDefault)
-          , testGroup "roundTrip" [
-                testProperty "string"                (toParseJSON thApproxParseJSONUnwrap  thApproxToJSONUnwrap)
-              , testProperty "ObjectWithSingleField" (toParseJSON thApproxParseJSONDefault thApproxToJSONDefault)
-            ]
+        ]
+      , testGroup "Approx" [
+          testProperty "string"                (isString                . thApproxToJSONUnwrap)
+        , testProperty "ObjectWithSingleField" (isObjectWithSingleField . thApproxToJSONDefault)
+        , testGroup "roundTrip" [
+            testProperty "string"                (toParseJSON thApproxParseJSONUnwrap  thApproxToJSONUnwrap)
+          , testProperty "ObjectWithSingleField" (toParseJSON thApproxParseJSONDefault thApproxToJSONDefault)
           ]
-        , testGroup "GADT" [
-            testProperty "string"                (isString                . thGADTToJSONUnwrap)
-          , testProperty "ObjectWithSingleField" (isObjectWithSingleField . thGADTToJSONDefault)
-          , testGroup "roundTrip" [
-                testProperty "string"                (toParseJSON thGADTParseJSONUnwrap  thGADTToJSONUnwrap)
-              , testProperty "ObjectWithSingleField" (toParseJSON thGADTParseJSONDefault thGADTToJSONDefault)
-            ]
+        ]
+      , testGroup "GADT" [
+          testProperty "string"                (isString                . thGADTToJSONUnwrap)
+        , testProperty "ObjectWithSingleField" (isObjectWithSingleField . thGADTToJSONDefault)
+        , testGroup "roundTrip" [
+            testProperty "string"                (toParseJSON thGADTParseJSONUnwrap  thGADTToJSONUnwrap)
+          , testProperty "ObjectWithSingleField" (toParseJSON thGADTParseJSONDefault thGADTToJSONDefault)
+          ]
+        ]
+      , testGroup "OneConstructor" [
+          testProperty "default" (isEmptyArray . thOneConstructorToJSONDefault)
+        , testGroup "roundTrip" [
+            testProperty "default" (toParseJSON thOneConstructorParseJSONDefault thOneConstructorToJSONDefault)
           ]
         ]
       ]
@@ -462,6 +483,9 @@ tests = testGroup "properties" [
         thSomeTypeLiftToJSONObjectWithSingleField `sameAs1` thSomeTypeLiftToEncodingObjectWithSingleField
       , testProperty "SomeTypeObjectWithSingleField unary agree" $
         thSomeTypeToEncodingObjectWithSingleField `sameAs1Agree` thSomeTypeLiftToEncodingObjectWithSingleField
+
+      , testProperty "OneConstructor" $
+        thOneConstructorToJSONDefault `sameAs` thOneConstructorToEncodingDefault
       ]
     ]
   ]
