@@ -1,14 +1,11 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE DeriveGeneric #-}
-{-# LANGUAGE GADTs #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TemplateHaskell #-}
-#if __GLASGOW_HASKELL__ >= 708
-{-# LANGUAGE DataKinds #-}
-#endif
 
+-- For Data.Aeson.Types.camelTo
 {-# OPTIONS_GHC -fno-warn-deprecations #-}
 
 module UnitTests
@@ -20,61 +17,51 @@ module UnitTests
 import Prelude ()
 import Prelude.Compat
 
-import Control.Applicative (Const(..))
 import Control.Monad (forM, forM_)
-import Data.Aeson ((.=), (.:), (.:?), (.:!), FromJSON(..), FromJSONKeyFunction(..), FromJSONKey(..), ToJSON1(..), decode, eitherDecode, encode, genericParseJSON, genericToEncoding, genericToJSON, object, withObject)
+import Data.Aeson ((.=), (.:), (.:?), (.:!), FromJSON(..), FromJSONKeyFunction(..), FromJSONKey(..), ToJSON1(..), decode, eitherDecode, encode, genericToEncoding, genericToJSON, object, withObject)
 import Data.Aeson.Internal (JSONPathElement(..), formatError)
 import Data.Aeson.TH (deriveJSON, deriveToJSON, deriveToJSON1)
 import Data.Aeson.Text (encodeToTextBuilder)
-import Data.Aeson.Types (Options(..), SumEncoding(..), ToJSON(..), Value, camelTo, camelTo2, defaultOptions, omitNothingFields)
+import Data.Aeson.Types (Options(..), ToJSON(..), Value, camelTo, camelTo2, defaultOptions, omitNothingFields)
 import Data.Char (toUpper)
-import Data.Fixed (Pico)
-import Data.Functor.Compose (Compose(..))
-import Data.Functor.Identity (Identity(..))
-import Data.Functor.Product (Product(..))
-import Data.Functor.Sum (Sum(..))
+import Data.Either.Compat (isLeft, isRight)
 import Data.Hashable (hash)
-import Data.List.NonEmpty (NonEmpty(..))
+import Data.List (sort)
 import Data.Maybe (fromMaybe)
-import Data.Proxy (Proxy(..))
-import Data.Scientific (Scientific)
 import Data.Sequence (Seq)
 import Data.Tagged (Tagged(..))
 import Data.Text (Text)
 import Data.Time (UTCTime)
 import Data.Time.Format (parseTime)
 import Data.Time.Locale.Compat (defaultTimeLocale)
-import Data.Word (Word8)
 import GHC.Generics (Generic)
 import Instances ()
+import System.Directory (getDirectoryContents)
+import System.FilePath ((</>), takeExtension, takeFileName)
 import Test.Framework (Test, testGroup)
 import Test.Framework.Providers.HUnit (testCase)
-import Test.HUnit (Assertion, assertFailure, assertEqual)
+import Test.HUnit (Assertion, assertBool, assertFailure, assertEqual)
 import Text.Printf (printf)
-import Types (Approx(..), Compose3, Compose3', I)
 import UnitTests.NullaryConstructors (nullaryConstructors)
 import qualified Data.ByteString.Base16.Lazy as LBase16
 import qualified Data.ByteString.Lazy.Char8 as L
-import qualified Data.DList as DList
-import qualified Data.HashMap.Strict as HM
 import qualified Data.HashSet as HashSet
-import qualified Data.IntMap as IntMap
-import qualified Data.IntSet as IntSet
-import qualified Data.Map as M
-import qualified Data.Monoid as Monoid
-import qualified Data.Semigroup as Semigroup
-import qualified Data.Sequence as Seq
-import qualified Data.Set as Set
 import qualified Data.Text.Lazy as LT
 import qualified Data.Text.Lazy.Builder as TLB
 import qualified Data.Text.Lazy.Encoding as LT
 import qualified Data.Text.Lazy.Encoding as TLE
-import qualified Data.Tree as Tree
-import qualified Data.Vector as Vector
+import qualified ErrorMessages
+import qualified SerializationFormatSpec
+
+-- Asserts that we can use both modules at once in the test suite.
+import Data.Aeson.Parser.UnescapeFFI ()
+import Data.Aeson.Parser.UnescapePure ()
 
 tests :: Test
 tests = testGroup "unit" [
-    testGroup "camelCase" [
+    testGroup "SerializationFormatSpec" SerializationFormatSpec.tests
+  , testGroup "ErrorMessages" ErrorMessages.tests
+  , testGroup "camelCase" [
       testCase "camelTo" $ roundTripCamel "aName"
     , testCase "camelTo" $ roundTripCamel "another"
     , testCase "camelTo" $ roundTripCamel "someOtherName"
@@ -87,19 +74,16 @@ tests = testGroup "unit" [
         assertEqual "" "camel_api_case" (camelTo2 '_' "CamelAPICase")
     ]
   , testGroup "encoding" [
-      testCase "goodProducer" $ goodProducer
+      testCase "goodProducer" goodProducer
     ]
   , testGroup "utctime" [
-      testCase "good" $ utcTimeGood
-    , testCase "bad"  $ utcTimeBad
+      testCase "good" utcTimeGood
+    , testCase "bad"  utcTimeBad
     ]
   , testGroup "formatError" [
-      testCase "example 1" $ formatErrorExample
+      testCase "example 1" formatErrorExample
     ]
   , testGroup ".:, .:?, .:!" $ fmap (testCase "-") dotColonMark
-  , testGroup "To JSON representation" $ fmap assertJsonEncodingExample jsonEncodingExamples
-  , testGroup "From JSON representation" $ fmap assertJsonExample jsonDecodingExamples
-  , testGroup "To/From JSON representation" $ fmap assertJsonExample jsonExamples
   , testGroup "JSONPath" $ fmap (testCase "-") jsonPath
   , testGroup "Hashable laws" $ fmap (testCase "-") hashableLaws
   , testGroup "Issue #351" $ fmap (testCase "-") issue351
@@ -107,6 +91,7 @@ tests = testGroup "unit" [
   , testGroup "FromJSONKey" $ fmap (testCase "-") fromJSONKeyAssertions
   , testCase "PR #455" pr455
   , testCase "Unescape string (PR #477)" unescapeString
+  , testCase "Show Options" showOptions
   ]
 
 roundTripCamel :: String -> Assertion
@@ -123,17 +108,6 @@ camelFrom c s = let (p:ps) = split c s
     capitalize t = toUpper (head t) : tail t
 
 
-data MyEither a b = MyLeft a | MyRight b
-  deriving (Generic, Show, Eq)
-
-instance (ToJSON a, ToJSON b) => ToJSON (MyEither a b) where
-    toJSON = genericToJSON defaultOptions { sumEncoding = UntaggedValue }
-    toEncoding = genericToEncoding defaultOptions { sumEncoding = UntaggedValue }
-
-instance (FromJSON a, FromJSON b) => FromJSON (MyEither a b) where
-    parseJSON = genericParseJSON defaultOptions { sumEncoding = UntaggedValue }
-
-
 data Wibble = Wibble {
     wibbleString :: String
   , wibbleInt :: Int
@@ -213,11 +187,9 @@ utcTimeGood = do
   where
     parseWithRead :: String -> LT.Text -> UTCTime
     parseWithRead f s =
-      case parseTime defaultTimeLocale f . LT.unpack $ s of
-        Nothing -> error "parseTime input malformed"
-        Just t  -> t
+      fromMaybe (error "parseTime input malformed") . parseTime defaultTimeLocale f . LT.unpack $ s
     parseWithAeson :: LT.Text -> Maybe UTCTime
-    parseWithAeson s = decode . LT.encodeUtf8 $ (LT.concat ["\"", s, "\""])
+    parseWithAeson s = decode . LT.encodeUtf8 $ LT.concat ["\"", s, "\""]
 
 -- Test that a few non-timezone qualified timestamp formats get
 -- rejected if decoding to UTCTime.
@@ -233,7 +205,7 @@ utcTimeBad = do
   verifyFailParse "2015-01-03 12:13.000Z" -- decimal at the end, but no seconds
   where
     verifyFailParse (s :: LT.Text) =
-      let (dec :: Maybe UTCTime) = decode . LT.encodeUtf8 $ (LT.concat ["\"", s, "\""]) in
+      let (dec :: Maybe UTCTime) = decode . LT.encodeUtf8 $ LT.concat ["\"", s, "\""] in
       assertEqual "verify failure" Nothing dec
 
 -- Non identifier keys should be escaped & enclosed in brackets
@@ -261,11 +233,11 @@ dotColonMark = [
   , assertEqual ".:  42"          (Just (T1 (Just 42))) (decode ex2 :: Maybe T1)
   , assertEqual ".:  null"        (Just (T1 Nothing))   (decode ex3 :: Maybe T1)
 
-  , assertEqual ".:? not-present" (Just (T2 (Nothing))) (decode ex1 :: Maybe T2)
+  , assertEqual ".:? not-present" (Just (T2 Nothing))   (decode ex1 :: Maybe T2)
   , assertEqual ".:? 42"          (Just (T2 (Just 42))) (decode ex2 :: Maybe T2)
   , assertEqual ".:? null"        (Just (T2 Nothing))   (decode ex3 :: Maybe T2)
 
-  , assertEqual ".:! not-present" (Just (T3 (Nothing))) (decode ex1 :: Maybe T3)
+  , assertEqual ".:! not-present" (Just (T3 Nothing))   (decode ex1 :: Maybe T3)
   , assertEqual ".:! 42"          (Just (T3 (Just 42))) (decode ex2 :: Maybe T3)
   , assertEqual ".:! null"        Nothing               (decode ex3 :: Maybe T3)
   ]
@@ -273,173 +245,6 @@ dotColonMark = [
         ex2 = "{\"value\": 42 }"
         ex3 = "{\"value\": null }"
 
-------------------------------------------------------------------------------
--- These tests assert that the JSON serialization doesn't change by accident.
------------------------------------------------------------------------------
-
-data Example where
-    Example
-        :: (Eq a, Show a, ToJSON a, FromJSON a)
-        => String -> L.ByteString -> a -> Example
-    MaybeExample
-        :: (Eq a, Show a, FromJSON a)
-        => String -> L.ByteString -> Maybe a -> Example
-
-assertJsonExample :: Example -> Test
-assertJsonExample (Example name bs val) = testCase name $ do
-    assertEqual "encode"           bs         (encode val)
-    assertEqual "encode/via value" bs         (encode $ toJSON val)
-    assertEqual "decode"           (Just val) (decode bs)
-assertJsonExample (MaybeExample name bs mval) = testCase name $
-    assertEqual "decode" mval (decode bs)
-
-assertJsonEncodingExample :: Example -> Test
-assertJsonEncodingExample (Example name bs val) = testCase name $ do
-    assertEqual "encode"           bs (encode val)
-    assertEqual "encode/via value" bs (encode $ toJSON val)
-assertJsonEncodingExample (MaybeExample name _ _) = testCase name $
-    assertFailure "cannot encode MaybeExample"
-
-jsonEncodingExamples :: [Example]
-jsonEncodingExamples =
-  [
-  -- Maybe serialising is lossy
-  -- https://github.com/bos/aeson/issues/376
-    Example "Just Nothing" "null" (Just Nothing :: Maybe (Maybe Int))
-  -- infinities cannot be recovered, null is decoded as NaN
-  , Example "inf :: Double" "null" (Approx $ 1/0 :: Approx Double)
-  ]
-
-jsonDecodingExamples :: [Example]
-jsonDecodingExamples = [
-  -- Maybe serialising is lossy
-  -- https://github.com/bos/aeson/issues/376
-    MaybeExample "Nothing"      "null" (Just $ Nothing :: Maybe (Maybe Int))
-  , MaybeExample "Just"         "1"    (Just $ Just 1 :: Maybe (Maybe Int))
-  , MaybeExample "Just Nothing" "null" (Just $ Nothing :: Maybe (Maybe (Maybe Int)))
-  -- Integral values are truncated, and overflowed
-  -- https://github.com/bos/aeson/issues/317
-  , MaybeExample "Word8 3"    "3"    (Just 3 :: Maybe Word8)
-  , MaybeExample "Word8 3.00" "3.00" (Just 3 :: Maybe Word8)
-  , MaybeExample "Word8 3.14" "3.14" (Nothing :: Maybe Word8)
-  , MaybeExample "Word8 -1"   "-1"   (Nothing :: Maybe Word8)
-  , MaybeExample "Word8 300"  "300"  (Nothing :: Maybe Word8)
-  ]
-
-jsonExamples :: [Example]
-jsonExamples =
-  [
-    Example "Either Left" "{\"Left\":1}"  (Left 1 :: Either Int Int)
-  , Example "Either Right" "{\"Right\":1}"  (Right 1 :: Either Int Int)
-  , Example "Nothing"  "null"  (Nothing :: Maybe Int)
-  , Example "Just"  "1"  (Just 1 :: Maybe Int)
-  , Example "Proxy Int" "null"  (Proxy :: Proxy Int)
-  , Example "Tagged Char Int" "1"  (Tagged 1 :: Tagged Char Int)
-#if __GLASGOW_HASKELL__ >= 708
-    -- Test Tagged instance is polykinded
-  , Example "Tagged 123 Int" "1"  (Tagged 1 :: Tagged 123 Int)
-#endif
-  , Example "Const Char Int" "\"c\""  (Const 'c' :: Const Char Int)
-  , Example "Tuple" "[1,2]"  ((1, 2) :: (Int, Int))
-  , Example "NonEmpty" "[1,2,3]"  (1 :| [2, 3] :: NonEmpty Int)
-  , Example "Seq" "[1,2,3]"  (Seq.fromList [1, 2, 3] ::  Seq.Seq Int)
-  , Example "DList" "[1,2,3]"  (DList.fromList [1, 2, 3] :: DList.DList Int)
-  , Example "()" "[]"  ()
-
-  , Example "HashMap Int Int"          "{\"0\":1,\"2\":3}"  (HM.fromList [(0,1),(2,3)] :: HM.HashMap Int Int)
-  , Example "Map Int Int"              "{\"0\":1,\"2\":3}"  (M.fromList [(0,1),(2,3)] :: M.Map Int Int)
-  , Example "Map (Tagged Int Int) Int" "{\"0\":1,\"2\":3}"  (M.fromList [(Tagged 0,1),(Tagged 2,3)] :: M.Map (Tagged Int Int) Int)
-  , Example "Map [Int] Int"            "[[[0],1],[[2],3]]"  (M.fromList [([0],1),([2],3)] :: M.Map [Int] Int)
-  , Example "Map [Char] Int"           "{\"ab\":1,\"cd\":3}"  (M.fromList [("ab",1),("cd",3)] :: M.Map [Char] Int)
-  , Example "Map [I Char] Int"         "{\"ab\":1,\"cd\":3}"  (M.fromList [(map pure "ab",1),(map pure "cd",3)] :: M.Map [I Char] Int)
-
-  , Example "nan :: Double" "null"  (Approx $ 0/0 :: Approx Double)
-
-  , Example "Ordering LT" "\"LT\"" LT
-  , Example "Ordering EQ" "\"EQ\"" EQ
-  , Example "Ordering GT" "\"GT\"" GT
-
-  , Example "Float" "3.14" (3.14 :: Float)
-  , Example "Pico" "3.14" (3.14 :: Pico)
-  , Example "Scientific" "3.14" (3.14 :: Scientific)
-
-  , Example "Set Int" "[1,2,3]" (Set.fromList [3, 2, 1] :: Set.Set Int)
-  , Example "IntSet"  "[1,2,3]" (IntSet.fromList [3, 2, 1])
-  , Example "IntMap" "[[1,2],[3,4]]" (IntMap.fromList [(3,4), (1,2)] :: IntMap.IntMap Int)
-  , Example "Vector" "[1,2,3]" (Vector.fromList [1, 2, 3] :: Vector.Vector Int)
-  , Example "HashSet Int" "[1,2,3]" (HashSet.fromList [3, 2, 1] :: HashSet.HashSet Int)
-  , Example "Tree Int" "[1,[[2,[[3,[]],[4,[]]]],[5,[]]]]" (let n = Tree.Node in n 1 [n 2 [n 3 [], n 4 []], n 5 []] :: Tree.Tree Int)
-
-  -- Three separate cases, as ordering in HashMap is not defined
-  , Example "HashMap Float Int, NaN" "{\"NaN\":1}"  (Approx $ HM.singleton (0/0) 1 :: Approx (HM.HashMap Float Int))
-  , Example "HashMap Float Int, Infinity" "{\"Infinity\":1}"  (HM.singleton (1/0) 1 :: HM.HashMap Float Int)
-  , Example "HashMap Float Int, +Infinity" "{\"-Infinity\":1}"  (HM.singleton (negate 1/0) 1 :: HM.HashMap Float Int)
-
-  -- Functors
-  , Example "Identity Int" "1"  (pure 1 :: Identity Int)
-
-  , Example "Identity Char" "\"x\""      (pure 'x' :: Identity Char)
-  , Example "Identity String" "\"foo\""  (pure "foo" :: Identity String)
-  , Example "[Identity Char]" "\"xy\""   ([pure 'x', pure 'y'] :: [Identity Char])
-
-  , Example "Maybe Char" "\"x\""              (pure 'x' :: Maybe Char)
-  , Example "Maybe String" "\"foo\""          (pure "foo" :: Maybe String)
-  , Example "Maybe [Identity Char]" "\"xy\""  (pure [pure 'x', pure 'y'] :: Maybe [Identity Char])
-
-  , Example "Product I Maybe Int" "[1,2]"         (Pair (pure 1) (pure 2) :: Product I Maybe Int)
-  , Example "Product I Maybe Int" "[1,null]"      (Pair (pure 1) Nothing :: Product I Maybe Int)
-  , Example "Product I [] Char" "[\"a\",\"foo\"]" (Pair (pure 'a') "foo" :: Product I [] Char)
-
-  , Example "Sum I [] Int: InL"  "{\"InL\":1}"       (InL (pure 1) :: Sum I [] Int)
-  , Example "Sum I [] Int: InR"  "{\"InR\":[1,2]}"   (InR [1, 2] :: Sum I [] Int)
-  , Example "Sum I [] Char: InR" "{\"InR\":\"foo\"}" (InR "foo" :: Sum I [] Char)
-
-  , Example "Compose I  I  Int" "1"      (pure 1 :: Compose I I   Int)
-  , Example "Compose I  [] Int" "[1]"    (pure 1 :: Compose I []  Int)
-  , Example "Compose [] I  Int" "[1]"    (pure 1 :: Compose [] I  Int)
-  , Example "Compose [] [] Int" "[[1]]"  (pure 1 :: Compose [] [] Int)
-
-  , Example "Compose I  I  Char" "\"x\""    (pure 'x' :: Compose I  I  Char)
-  , Example "Compose I  [] Char" "\"x\""    (pure 'x' :: Compose I  [] Char)
-  , Example "Compose [] I  Char" "\"x\""    (pure 'x' :: Compose [] I  Char)
-  , Example "Compose [] [] Char" "[\"x\"]"  (pure 'x' :: Compose [] [] Char)
-
-  , Example "Compose3 I  I  I  Char" "\"x\""      (pure 'x' :: Compose3 I  I  I  Char)
-  , Example "Compose3 I  I  [] Char" "\"x\""      (pure 'x' :: Compose3 I  I  [] Char)
-  , Example "Compose3 I  [] I  Char" "\"x\""      (pure 'x' :: Compose3 I  [] I  Char)
-  , Example "Compose3 I  [] [] Char" "[\"x\"]"    (pure 'x' :: Compose3 I  [] [] Char)
-  , Example "Compose3 [] I  I  Char" "\"x\""      (pure 'x' :: Compose3 [] I  I  Char)
-  , Example "Compose3 [] I  [] Char" "[\"x\"]"    (pure 'x' :: Compose3 [] I  [] Char)
-  , Example "Compose3 [] [] I  Char" "[\"x\"]"    (pure 'x' :: Compose3 [] [] I  Char)
-  , Example "Compose3 [] [] [] Char" "[[\"x\"]]"  (pure 'x' :: Compose3 [] [] [] Char)
-
-  , Example "Compose3' I  I  I  Char" "\"x\""      (pure 'x' :: Compose3' I  I  I  Char)
-  , Example "Compose3' I  I  [] Char" "\"x\""      (pure 'x' :: Compose3' I  I  [] Char)
-  , Example "Compose3' I  [] I  Char" "\"x\""      (pure 'x' :: Compose3' I  [] I  Char)
-  , Example "Compose3' I  [] [] Char" "[\"x\"]"    (pure 'x' :: Compose3' I  [] [] Char)
-  , Example "Compose3' [] I  I  Char" "\"x\""      (pure 'x' :: Compose3' [] I  I  Char)
-  , Example "Compose3' [] I  [] Char" "[\"x\"]"    (pure 'x' :: Compose3' [] I  [] Char)
-  , Example "Compose3' [] [] I  Char" "[\"x\"]"    (pure 'x' :: Compose3' [] [] I  Char)
-  , Example "Compose3' [] [] [] Char" "[[\"x\"]]"  (pure 'x' :: Compose3' [] [] [] Char)
-
-  , Example "MyEither Int String: Left"  "42"      (MyLeft 42     :: MyEither Int String)
-  , Example "MyEither Int String: Right" "\"foo\"" (MyRight "foo" :: MyEither Int String)
-
-  -- newtypes from Monoid/Semigroup
-  , Example "Monoid.Dual Int" "2" (pure 2 :: Monoid.Dual Int)
-  , Example "Monoid.First Int" "2" (pure 2 :: Monoid.First Int)
-  , Example "Monoid.Last Int" "2" (pure 2 :: Monoid.Last Int)
-  , Example "Semigroup.Min Int" "2" (pure 2 :: Semigroup.Min Int)
-  , Example "Semigroup.Max Int" "2" (pure 2 :: Semigroup.Max Int)
-  , Example "Semigroup.First Int" "2" (pure 2 :: Semigroup.First Int)
-  , Example "Semigroup.Last Int" "2" (pure 2 :: Semigroup.Last Int)
-  , Example "Semigroup.WrappedMonoid Int" "2" (Semigroup.WrapMonoid 2 :: Semigroup.WrappedMonoid Int)
-  , Example "Semigroup.Option Just" "2" (pure 2 :: Semigroup.Option Int)
-  , Example "Semigroup.Option Nothing" "null" (Semigroup.Option (Nothing :: Maybe Bool))
-  ]
-
-
-
 ------------------------------------------------------------------------------
 -- These tests check that JSONPath is tracked correctly
 -----------------------------------------------------------------------------
@@ -536,7 +341,8 @@ issue351 = [
 ioTests :: IO [Test]
 ioTests = do
   enc <- encoderComparisonTests
-  return [enc]
+  js <- jsonTestSuite
+  return [enc, js]
 
 encoderComparisonTests :: IO Test
 encoderComparisonTests = do
@@ -596,6 +402,76 @@ unescapeString = do
           L.concat ["\"\\u", L.take 4 s, "\\u", L.drop 4 s, "\""]
       | otherwise = error "unescapeString: can't happen"
 
+-- JSONTestSuite
+
+jsonTestSuiteTest :: FilePath -> Test
+jsonTestSuiteTest path = testCase fileName $ do
+    payload <- L.readFile path
+    let result = eitherDecode payload :: Either String Value
+    assertBool fileName $ case take 2 fileName of
+      "i_" -> isRight result
+      "n_" -> isLeft result
+      "y_" -> isRight result
+      _    -> isRight result -- test_transform tests have inconsistent names
+  where
+    fileName = takeFileName path
+
+-- Build a collection of tests based on the current contents of the
+-- JSONTestSuite test directories.
+
+jsonTestSuite :: IO Test
+jsonTestSuite = do
+  let suitePath = "tests/JSONTestSuite"
+  let suites = ["test_parsing", "test_transform"]
+  testPaths <- fmap (sort . concat) . forM suites $ \suite -> do
+    let dir = suitePath </> suite
+    entries <- getDirectoryContents dir
+    let ok name = takeExtension name == ".json" &&
+                  not (name `HashSet.member` blacklist)
+    return . map (dir </>) . filter ok $ entries
+  return $ testGroup "JSONTestSuite" $ map jsonTestSuiteTest testPaths
+
+-- The set expected-to-be-failing JSONTestSuite tests.
+-- Not all of these failures are genuine bugs.
+-- Of those that are bugs, not all are worth fixing.
+
+blacklist :: HashSet.HashSet String
+-- blacklist = HashSet.empty
+blacklist = _blacklist
+
+_blacklist :: HashSet.HashSet String
+_blacklist = HashSet.fromList [
+    "i_object_key_lone_2nd_surrogate.json"
+  , "i_string_1st_surrogate_but_2nd_missing.json"
+  , "i_string_1st_valid_surrogate_2nd_invalid.json"
+  , "i_string_UTF-16LE_with_BOM.json"
+  , "i_string_UTF-16_invalid_lonely_surrogate.json"
+  , "i_string_UTF-16_invalid_surrogate.json"
+  , "i_string_UTF-8_invalid_sequence.json"
+  , "i_string_incomplete_surrogate_and_escape_valid.json"
+  , "i_string_incomplete_surrogate_pair.json"
+  , "i_string_incomplete_surrogates_escape_valid.json"
+  , "i_string_invalid_lonely_surrogate.json"
+  , "i_string_invalid_surrogate.json"
+  , "i_string_inverted_surrogates_U+1D11E.json"
+  , "i_string_lone_second_surrogate.json"
+  , "i_string_not_in_unicode_range.json"
+  , "i_string_truncated-utf-8.json"
+  , "i_structure_UTF-8_BOM_empty_object.json"
+  , "n_string_unescaped_crtl_char.json"
+  , "n_string_unescaped_newline.json"
+  , "n_string_unescaped_tab.json"
+  , "string_1_escaped_invalid_codepoint.json"
+  , "string_1_invalid_codepoint.json"
+  , "string_1_invalid_codepoints.json"
+  , "string_2_escaped_invalid_codepoints.json"
+  , "string_2_invalid_codepoints.json"
+  , "string_3_escaped_invalid_codepoints.json"
+  , "string_3_invalid_codepoints.json"
+  , "y_string_utf16BE_no_BOM.json"
+  , "y_string_utf16LE_no_BOM.json"
+  ]
+
 -- A regression test for: https://github.com/bos/aeson/pull/455
 data Foo a = FooNil | FooCons (Foo Int)
 
@@ -606,6 +482,20 @@ pr455 = assertEqual "FooCons FooNil"
     foo :: Foo Int
     foo = FooCons FooNil
 
+showOptions :: Assertion
+showOptions =
+    assertEqual
+        "Show Options"
+        (  "Options {"
+        ++   "fieldLabelModifier =~ \"exampleField\""
+        ++ ", constructorTagModifier =~ \"ExampleConstructor\""
+        ++ ", allNullaryToStringTag = True"
+        ++ ", omitNothingFields = False"
+        ++ ", sumEncoding = TaggedObject {tagFieldName = \"tag\", contentsFieldName = \"contents\"}"
+        ++ ", unwrapUnaryRecords = False"
+        ++ "}")
+        (show defaultOptions)
+
 deriveJSON defaultOptions{omitNothingFields=True} ''MyRecord
 
 deriveToJSON  defaultOptions ''Foo
